using System;
using System.Collections.Generic;
using System.Text;
using System.Reflection;

namespace MySample.Web
{
    public class JsonSerializer
    {
        #region Static members
        private static Dictionary<string, Dictionary<string, PropertyInfo>> _cache = new Dictionary<string, Dictionary<string, PropertyInfo>>();
        #endregion
        #region Constructors
        /// <summary>
        /// Create JSON serializer
        /// </summary>
        /// <param name="objectType">Object type to serialize</param>
        public JsonSerializer(Type objectType) : this(objectType, '\'')
        {}
        public JsonSerializer(Type objectType, char delimiter)
        {
            _objectType = objectType;
            _stringDelimiter = delimiter;
            CacheType();
        }
        #endregion
        #region Public properties
        #region --> ObjectType
        private Type _objectType;
        /// <summary>
        /// Get object type
        /// </summary>
        public Type ObjectType
        {
            get { return _objectType; }
        }
        #endregion
        #region --> StringDelimiter
        private char _stringDelimiter;
        /// <summary>
        /// Get or set string delimiter
        /// </summary>
        public char StringDelimiter
        {
            get { return _stringDelimiter; }
            set { _stringDelimiter = value; }
        }
        #endregion
        #endregion
        #region Private methods
        private void CacheType()
        {
            if (!JsonSerializer._cache.ContainsKey(_objectType.FullName))
            {
                Dictionary<string, PropertyInfo> jsonProperties = new Dictionary<string, PropertyInfo>();

                PropertyInfo[] properties = _objectType.GetProperties();
                if (properties != null && properties.Length > 0)
                {
                    foreach (PropertyInfo curProp in properties)
                    {
                        object[] customAttributes = curProp.GetCustomAttributes(typeof(JsonAttribute), true);
                        if (customAttributes != null && customAttributes.Length > 0)
                        {
                            JsonAttribute curAttr = customAttributes[0] as JsonAttribute;
                            jsonProperties.Add(curAttr.Name, curProp);
                        }

                    }
                }
                JsonSerializer._cache.Add(_objectType.FullName, jsonProperties);
            }
        }
        #endregion
        #region Public methods
        /// <summary>
        /// Serialize object in JSON notation
        /// </summary>
        /// <param name="toSerialize">object to serialize</param>
        /// <returns>string ready for javascript injection</returns>
        public string Serialize(object toSerialize)
        {
            StringBuilder sb = new StringBuilder();
            bool firstOccurrence = true;
            sb.Append("{");
            
            // Get properties
            Dictionary<string, PropertyInfo> jsonProperties = JsonSerializer._cache[_objectType.FullName];
            
            foreach (string jsonPropertyName in jsonProperties.Keys)            
            {
                // Check delimiter
                if (!firstOccurrence)
                    sb.Append(",");
                else
                    firstOccurrence = false;
                
                // Set property name
                sb.Append(_stringDelimiter); sb.Append(jsonPropertyName); sb.Append(_stringDelimiter);

                // Set property value
                PropertyInfo propInfo = jsonProperties[jsonPropertyName];
                if ( propInfo.PropertyType == typeof(string))
                {
                    sb.Append(" : ");
                    sb.Append(_stringDelimiter); 
                    sb.Append(propInfo.GetValue(toSerialize, null)); 
                    sb.Append(_stringDelimiter);
                }
            }

            sb.Append("}");
            return sb.ToString();
        }
        #endregion
    }
}
